@@ -9,4 +9,7 @@ text4 = '猫'
 
 print(num1, type(num1))
 assert(num1 == 42)
-num2 = 42
\ No newline at end of file
+num2 = 42
+
+assert(bool1)
+assert(not bool2)
\ No newline at end of file
